#include "stdafx.h"
#include "XCPClockDlg.h"
#include "TitleMenu.h"
#include "about.h"
#include "TransparencyDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else if ((nID & 0xFFF0) == SC_CLOSE)
	{
		OnClose();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnAbout()
{
	CAboutDlg dlgAbout;
	dlgAbout.DoModal();
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::ShowPopup(CPoint point)
{
	int i = 0;

	UINT nFlags = MF_STRING;

	CTitleMenu popup;
	VERIFY(popup.CreatePopupMenu());
	popup.AddMenuTitle(_T("The CodeProject Clock"));
	popup.SetColor(RGB(255,153,0));
	popup.SetGradientColor(CTitleMenu::LightenColor(RGB(255,153,0), .7));
	popup.SetTextColor(RGB(0,0,255));

	///////////////////////////////////////////////////////////////////////////
	// skins

	CMenu skins_popup;
	VERIFY(skins_popup.CreatePopupMenu());

	for (i = 0; m_Skins[i].nID != 0; i++)
	{
		nFlags = MF_STRING;
		if (i == m_eSkin)
			nFlags |= MF_CHECKED;
		VERIFY(skins_popup.AppendMenu(nFlags, m_Skins[i].nID, m_Skins[i].pszName));
	}

	CString s = _T("");

	s.Format(_T("Skins (%s)"), m_Skins[m_eSkin].pszName);
	VERIFY(popup.AppendMenu(MF_STRING|MF_POPUP, (UINT)(HMENU)skins_popup, s));

	s.Format(_T("Transparency (%u) ..."), m_nTransparency);
	VERIFY(popup.AppendMenu(MF_STRING, ID_TRANSPARENCY, s));

	nFlags = MF_STRING;
	if (m_bAlwaysOnTop)
		nFlags |= MF_CHECKED;
	VERIFY(popup.AppendMenu(nFlags, ID_ALWAYS_ON_TOP, _T("Always On Top")));

	nFlags = MF_STRING;
	if (m_bShowSecondHand)
		nFlags |= MF_CHECKED;
	VERIFY(popup.AppendMenu(nFlags, ID_SHOW_SECOND_HAND, _T("Show Second Hand")));

	VERIFY(popup.AppendMenu(MF_SEPARATOR)); //----------------------------------------------------

	VERIFY(popup.AppendMenu(MF_STRING, ID_ONLINE_HELP, _T("Online Help...")));
	VERIFY(popup.AppendMenu(MF_STRING, ID_APP_ABOUT, _T("About XCPClock...")));

	VERIFY(popup.AppendMenu(MF_SEPARATOR)); //----------------------------------------------------

	VERIFY(popup.AppendMenu(MF_STRING, ID_CLOSE_ALL, _T("Close All XCPClock")));
	VERIFY(popup.AppendMenu(MF_STRING, ID_EXIT, _T("Exit")));

	// display the menu

	CPoint pt = point;
	ClientToScreen(&pt);
	VERIFY(popup.TrackPopupMenu(TPM_LEFTALIGN | TPM_LEFTBUTTON, pt.x+1, pt.y+1, this));
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnAlwaysOnTop()
{
	m_bAlwaysOnTop = !m_bAlwaysOnTop;
	
	if (m_bAlwaysOnTop)
		SetWindowPos(&wndTopMost, 0,0,0,0, SWP_NOMOVE|SWP_NOSIZE);
	else
		SetWindowPos(&wndNoTopMost, 0,0,0,0, SWP_NOMOVE|SWP_NOSIZE);
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnShowSecondHand()
{
	m_bShowSecondHand = !m_bShowSecondHand;
	m_Clock.ShowSecondHand(m_bShowSecondHand);
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnSkin(UINT nID)
{
	m_eSkin = (SKIN) (nID - ID_SKIN_CPBOB);
	SetSkin();
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnTransparency()
{
	CTransparencyDialog dlg;
	dlg.m_nByteTransparency = m_nTransparency;
	if (dlg.DoModal() == IDOK)
	{
		m_nTransparency = dlg.m_nByteTransparency;
		SetTransparentColor(RGB(255,0,255), TRUE, m_nTransparency);
	}
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnOnlineHelp() 
{
	CString strHelp = _T("");
	VERIFY(strHelp.LoadString(IDS_ONLINE_HELP));
	CXHyperLink::GotoURL(strHelp, SW_SHOW, TRUE);	
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnIncreaseTransparency() 
{
	TRACE(_T("in CXCPClockDlg::OnIncreaseTransparency\n"));
	if (m_nTransparency <= 245)
	{
		m_nTransparency += 10;
		SetTransparentColor(RGB(255,0,255), TRUE, m_nTransparency);
	}
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnDecreaseTransparency() 
{
	TRACE(_T("in CXCPClockDlg::OnDecreaseTransparency\n"));
	if (m_nTransparency >= 40)
	{
		m_nTransparency -= 10;
		SetTransparentColor(RGB(255,0,255), TRUE, m_nTransparency);
	}
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnAdvanceHour() 
{
#ifdef _DEBUG
	CTime t = m_Clock.GetTime();
	int nHour = t.GetHour();
	nHour += 1;
	if (nHour > 23)
		nHour = 0;
	t = CTime(t.GetYear(), t.GetMonth(), t.GetDay(), nHour, t.GetMinute(), 59);
	m_Clock.UseSystemTime(FALSE);
	m_Clock.SetTime(t);
	m_Clock.RedrawWindow();
#endif
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnFreeze() 
{
#ifdef _DEBUG
	// The purpose of this is to facilitate screenshots
	m_Clock.ToggleDebug();
#endif
}
