// XCPClockDlg.cpp  Version 1.1 - see article at www.CodeProject.com
//
// Author:  Hans Dietrich
//          hdietrich@gmail.com
//
// Description:
//     XCPClockDlg implements the dialog for XCPClock.
//
// History
//     Version 1.1 - 2008 August 23
//     - Minor bug fixes
//     - Changed to automatic link to <i>msimg32.lib</i> (previously you had to 
//       add this lib to the project link modules)
//     - Added new skins
//     - Added click-thru when transparency is enabled (less than 255)
//     - Added shortcuts for setting transparency
//     - Added automatic hand color for new transparent clock faces
//     - Added command line options
//     - Added synchronized move
//     - Added global close option to popup menu
//     - Added option to show/hide second hand
//     - Added VS2005 project
//
//     Version 1.0 - 2006 March 23
//     - Initial public release
//
// License:
//     This software is released into the public domain.  You are free to use
//     it in any way you like, except that you may not sell this source code.
//
//     This software is provided "as is" with no expressed or implied warranty.
//     I accept no liability for any damage or loss of business that this
//     software may cause.
//
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "XCPClock.h"
#include "XCPClockDlg.h"
#include "about.h"
#include "TitleMenu.h"
#include "WindowPos.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// these variables should have been defined in some standard header but are not
#ifndef WS_EX_LAYERED
#define WS_EX_LAYERED 0x00080000 
#endif
#ifndef LWA_COLORKEY
#define LWA_COLORKEY	1	// Use color as the transparency color.
#endif
#ifndef LWA_ALPHA
#define LWA_ALPHA		2	// Use bAlpha to determine the opacity of the layer
#endif

typedef BOOL (WINAPI *lpfnSetLayeredWindowAttributes)
				(HWND hWnd, COLORREF cr, BYTE bAlpha, DWORD dwFlags);

static lpfnSetLayeredWindowAttributes g_pSetLayeredWindowAttributes = NULL;
static CPoint g_pointScreen = CPoint(0, 0);

static UINT WM_XCPCLOCK_INIT = ::RegisterWindowMessage(_T("WM_XCPCLOCK_INIT"));

///////////////////////////////////////////////////////////////////////////////
// skin definitions - must be in same order as ID_SKIN_xxxx definitions
//
CXCPClockDlg::SKIN_MENU_DATA CXCPClockDlg::m_Skins[] =
{
	ID_SKIN_CPBOB,         IDB_CPBOB_CLOCK_FACE,         IDB_SILVER_DATE,  _T("CP with Bob"),
	ID_SKIN_CP,            IDB_CP_CLOCK_FACE,            IDB_SILVER_DATE,  _T("CP without Bob"),
	ID_SKIN_DARKBLUE,      IDB_DARKBLUE_CLOCK_FACE,      IDB_BLUE_DATE,    _T("Dark Blue"),
	ID_SKIN_SILVER,        IDB_SILVER_CLOCK_FACE,        IDB_SILVER_DATE,  _T("Silver"),
	ID_SKIN_GOLD,          IDB_GOLD_CLOCK_FACE,          IDB_SILVER_DATE,  _T("Gold"),
	ID_SKIN_BLACK,         IDB_BLACK_CLOCK_FACE,         IDB_SILVER_DATE,  _T("Black"),
	ID_SKIN_LIGHTBLUE,     IDB_LIGHTBLUE_CLOCK_FACE,     IDB_BLUE_DATE,    _T("Light Blue"),
	ID_SKIN_GREEN,         IDB_GREEN_CLOCK_FACE,         IDB_SILVER_DATE,  _T("Green"),
	ID_SKIN_RED,           IDB_RED_CLOCK_FACE,           IDB_SILVER_DATE,  _T("Red"),
	ID_SKIN_ROSE,          IDB_ROSE_CLOCK_FACE,          IDB_SILVER_DATE,  _T("Rose"),
	ID_SKIN_AQUA,          IDB_AQUA_CLOCK_FACE,          IDB_SILVER_DATE,  _T("Aqua"),
	ID_SKIN_NEONBLUE,      IDB_NEONBLUE_CLOCK_FACE,      IDB_SILVER_DATE,  _T("Neon Blue"),
	ID_SKIN_BLUESTREAK,    IDB_BLUESTREAK_CLOCK_FACE,    IDB_SILVER_DATE,  _T("Blue Streak"),
	ID_SKIN_CRYSTALGREEN,  IDB_CRYSTALGREEN_CLOCK_FACE,  IDB_SILVER_DATE,  _T("Crystal Green"),
	ID_SKIN_CRYSTALBLACK,  IDB_CRYSTALBLACK_CLOCK_FACE,  IDB_SILVER_DATE,  _T("Crystal Black"),
	ID_SKIN_CRYSTALBLUE,   IDB_CRYSTALBLUE_CLOCK_FACE,   IDB_SILVER_DATE,  _T("Crystal Blue"),
	ID_SKIN_CRYSTALRED,    IDB_CRYSTALRED_CLOCK_FACE,    IDB_SILVER_DATE,  _T("Crystal Red"),
	ID_SKIN_CRYSTALYELLOW, IDB_CRYSTALYELLOW_CLOCK_FACE, IDB_SILVER_DATE,  _T("Crystal Yellow"),
	ID_SKIN_SILVERRING,    IDB_SILVERRING_CLOCK_FACE,    IDB_SILVER_DATE,  _T("Transparent Silver Ring"),
	ID_SKIN_MINIMAL,       IDB_MINIMAL_CLOCK_FACE,       IDB_SILVER_DATE,  _T("Transparent Minimal"),
	ID_SKIN_WHITE,         IDB_WHITE_CLOCK_FACE,         IDB_SILVER_DATE,  _T("White"),
	0,                     0,                            0,                0
};

int CXCPClockDlg::m_nSkins = sizeof(m_Skins)/sizeof(m_Skins[0]);

static CPtrArray g_hWndArray;


///////////////////////////////////////////////////////////////////////////////
// CXCPClockDlg dialog

BEGIN_MESSAGE_MAP(CXCPClockDlg, CDialog)
	//{{AFX_MSG_MAP(CXCPClockDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_ERASEBKGND()
	ON_WM_RBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_MOUSEWHEEL()
	ON_WM_DESTROY()
	ON_COMMAND(ID_EXIT, OnClose)
	ON_COMMAND(ID_ALWAYS_ON_TOP, OnAlwaysOnTop)
	ON_COMMAND(ID_SHOW_SECOND_HAND, OnShowSecondHand)
	ON_COMMAND(ID_TRANSPARENCY, OnTransparency)
	ON_COMMAND(ID_ONLINE_HELP, OnOnlineHelp)
	ON_COMMAND(ID_FREEZE, OnFreeze)
	ON_COMMAND(ID_CLOSE_ALL, OnCloseAll)
	ON_COMMAND(ID_ADVANCE_HOUR, OnAdvanceHour)
	ON_COMMAND(ID_INCREASE_TRANSPARENCY, OnIncreaseTransparency)
	ON_COMMAND(ID_DECREASE_TRANSPARENCY, OnDecreaseTransparency)
	ON_COMMAND(ID_APP_ABOUT, OnAbout)
	//}}AFX_MSG_MAP
	ON_COMMAND_RANGE(ID_SKIN_CPBOB, ID_SKIN_WHITE, OnSkin)
	ON_REGISTERED_MESSAGE(WM_XCPCLOCK_INIT, OnXCPClockInit)
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////////////////////////
CXCPClockDlg::CXCPClockDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CXCPClockDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CXCPClockDlg)
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_bMouseCaptured  = FALSE;
	m_bAlwaysOnTop    = FALSE;
	m_bShowSecondHand = TRUE;
	m_ptLButtonDown   = CPoint(0, 0);
	m_eSkin           = cpbob;
	m_nTransparency   = 255;

	HMODULE hUser32 = GetModuleHandle(_T("USER32.DLL"));
	g_pSetLayeredWindowAttributes = (lpfnSetLayeredWindowAttributes)
			GetProcAddress(hUser32, "SetLayeredWindowAttributes");
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CXCPClockDlg)
	DDX_Control(pDX, IDC_FRAME, m_Clock);
	//}}AFX_DATA_MAP
}

///////////////////////////////////////////////////////////////////////////////
BOOL CXCPClockDlg::OnInitDialog()
{
	ModifyStyleEx(WS_EX_APPWINDOW, 0);	// hide app from taskbar

	CDialog::OnInitDialog();

	m_hAccel = LoadAccelerators(AfxGetInstanceHandle(), 
					MAKEINTRESOURCE(IDR_ACCELERATOR));

	GetConfig();

	///////////////////////////////////////////////////////////////////////////

	SetTransparentColor(RGB(255,0,255), TRUE, m_nTransparency);

	// set resource IDs
	SetSkin();

	// set control size

	CSize size = m_Clock.GetWindowSize();

	CRect rect;
	m_Clock.GetWindowRect(&rect);
	ScreenToClient(&rect);
	rect.right = rect.left + size.cx;
	rect.bottom = rect.top + size.cy;
	m_Clock.MoveWindow(&rect);

	CRect rectDlg;
	GetWindowRect(&rectDlg);

	// set position

	CenterRectOnMonitor(rectDlg);

	CString strPos = _T("");
	strPos.Format(_T("0,1,%d,%d,%d,%d"), 
		rectDlg.left, rectDlg.top, rectDlg.right, rectDlg.bottom);

	RestoreWindowPos(m_hWnd, INI_PROGRAM_MAIN_WINDOW, NULL, strPos);

	if (!theApp.m_strToolTip.IsEmpty())
	{
		// tooltip will only display if started from command line
		TRACE(_T(",,,,, tooltip=<%s>\n"), theApp.m_strToolTip);
		m_ToolTip.Create(m_hWnd, AfxGetInstanceHandle());
	}

	///////////////////////////////////////////////////////////////////////////
	// command line args override registry values

	m_Clock.SetSecondsAdjust(theApp.m_nSecondsAdjust);

	if ((theApp.m_nX != BAD_XY) || (theApp.m_nY != BAD_XY))
	{
		CRect rect;
		GetWindowRect(&rect);
		int w = rect.Width();
		int h = rect.Height();
		if (theApp.m_nX != BAD_XY)
		{
			rect.left = theApp.m_nX;
			rect.right = rect.left + w;
		}
		if (theApp.m_nY != BAD_XY)
		{
			rect.top = theApp.m_nY;
			rect.bottom = rect.top + h;
		}
		TRACERECT(rect);
		MoveWindow(&rect);
	}

	if (!theApp.m_strSkin.IsEmpty())
	{
		m_eSkin = GetSkinFromName(theApp.m_strSkin);
		SetSkin();
	}

	//
	// set the name of the clock
	//
	m_Clock.SetName( theApp.m_strToolTip );
	m_Clock.SetNameColor( RGB( 255, 0, 0 ) );

	// start the clock
	m_Clock.Run();
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

///////////////////////////////////////////////////////////////////////////////
CXCPClockDlg::SKIN CXCPClockDlg::GetSkinFromName(LPCTSTR lpszSkinName)
{
	if ((m_eSkin < 0) || (m_eSkin >= m_nSkins))
	{
		TRACE(_T("ERROR - invalid skin %d\n"), m_eSkin);
		ASSERT(FALSE);
		m_eSkin = cpbob;
	}
	
	SKIN skin = m_eSkin;

	if (lpszSkinName && (lpszSkinName[0] != 0))
	{
		// these are the names that may be entered on command line -
		// must be in same order as m_Skins[] and ID_SKIN_xxxx definitions

		TCHAR * skins[] = 
		{
			_T("cpbob"),
			_T("cpnobob"),
			_T("darkblue"),
			_T("silver"),
			_T("gold"),
			_T("black"),
			_T("lightblue"),
			_T("green"),
			_T("red"),
			_T("rose"),
			_T("aqua"),
			_T("neonblue"),
			_T("bluestreak"),
			_T("crystalgreen"),
			_T("crystalblack"),
			_T("crystalblue"),
			_T("crystalred"),
			_T("crystalyellow"),
			_T("silverring"),
			_T("minimal"),
			_T("white"),
			0
		};

		for (int i = 0; skins[i] != 0; i++)
		{
			if (_tcsicmp(skins[i], theApp.m_strSkin) == 0)
			{
				skin = (SKIN) i;
				break;
			}
		}
	}

	return skin;
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::SetSkin()
{
	if ((m_eSkin < 0) || (m_eSkin >= m_nSkins))
	{
		TRACE(_T("ERROR - invalid skin %d\n"), m_eSkin);
		ASSERT(FALSE);
		m_eSkin = cpbob;
	}
	
	m_Clock.Stop();

	m_Clock.SetBitmaps(m_Skins[m_eSkin].nClockFaceBitmapId, 
		m_Skins[m_eSkin].nDateBitmapId);

	// set custom hand colors, depending on skin

	if (m_eSkin == neonblue)
	{
		m_Clock.SetHourHandColor(RGB(107,210,255))
			   .SetMinuteHandColor(RGB(107,210,255))
			   .SetSecondHandColor(RGB(254,80,90))
			   .SetAutomaticHandColor(FALSE);
	}
	else if (m_eSkin == crystalblue)
	{
		m_Clock.SetHourHandColor(RGB(0,0,139))
			   .SetMinuteHandColor(RGB(0,0,139))
			   .SetSecondHandColor(RGB(254,80,90))
			   .SetAutomaticHandColor(FALSE);
	}
	else if (m_eSkin == bluestreak)
	{
		m_Clock.SetHourHandColor(RGB(0,0,139))
			   .SetMinuteHandColor(RGB(0,0,139))
			   .SetSecondHandColor(RGB(254,80,90))
			   .SetAutomaticHandColor(FALSE);
	}
	else if (m_eSkin == crystalblack)
	{
		m_Clock.SetHourHandColor(RGB(255,255,255))
			   .SetMinuteHandColor(RGB(255,255,255))
			   .SetSecondHandColor(RGB(254,80,90))
			   .SetAutomaticHandColor(FALSE);
	}
	else if (m_eSkin == crystalred)
	{
		m_Clock.SetHourHandColor(RGB(255,255,95))
			   .SetMinuteHandColor(RGB(255,255,95))
			   .SetSecondHandColor(RGB(255,255,255))
			   .SetAutomaticHandColor(FALSE);
	}
	else if (m_eSkin == crystalyellow)
	{
		m_Clock.SetHourHandColor(RGB(80,90,172))
			   .SetMinuteHandColor(RGB(80,90,172))
			   .SetSecondHandColor(RGB(220,20,60))
			   .SetAutomaticHandColor(FALSE);
	}
	else if (m_eSkin == crystalgreen)
	{
		m_Clock.SetHourHandColor(RGB(0,100,0))
			   .SetMinuteHandColor(RGB(0,100,0))
			   .SetSecondHandColor(RGB(220,20,60))
			   .SetAutomaticHandColor(FALSE);
	}
	else if ((m_eSkin == silverring) || (m_eSkin == minimal))
	{
		m_Clock.SetHourHandColor(RGB(0,0,0))
			   .SetMinuteHandColor(RGB(0,0,0))
			   .SetSecondHandColor(RGB(220,20,60))
			   .SetAutomaticHandColor(TRUE);
	}
	else
	{
		m_Clock.SetHourHandColor(RGB(0,0,0))
			   .SetMinuteHandColor(RGB(0,0,0))
			   .SetSecondHandColor(RGB(220,20,60))
			   .SetAutomaticHandColor(FALSE);
	}

	// must repaint background tom ensure bitmap transparency
	Invalidate();

	m_Clock.Run();
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::GetConfig()
{
	TRACE(_T("in CXCPClockDlg::GetConfig\n"));

	// following are now read in XCPClock.cpp
	//m_bAlwaysOnTop   = theApp.GetProfileInt(INI_USER, INI_USER_ALWAYS_ON_TOP, FALSE);
	//m_nTransparency  = (BYTE)theApp.GetProfileInt(INI_USER, INI_USER_TRANSPARENCY, 255);

	m_bAlwaysOnTop = theApp.m_bAlwaysOnTop;
	m_bAlwaysOnTop = !m_bAlwaysOnTop;
	OnAlwaysOnTop();

	m_bShowSecondHand = theApp.GetProfileInt(INI_USER, INI_USER_SHOW_SECOND_HAND, TRUE);
	m_bShowSecondHand = !m_bShowSecondHand;
	OnShowSecondHand();

	m_nTransparency = theApp.m_nTransparency;

	int nSkin        = theApp.GetProfileInt(INI_USER, INI_USER_SKIN, 0);
	if ((nSkin < 0) || (nSkin >= m_nSkins))
		nSkin = 0;
	m_eSkin = (SKIN) nSkin;
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::SaveConfig()
{
	theApp.WriteProfileInt(INI_USER, INI_USER_ALWAYS_ON_TOP, m_bAlwaysOnTop);
	theApp.WriteProfileInt(INI_USER, INI_USER_SHOW_SECOND_HAND, m_bShowSecondHand);
	theApp.WriteProfileInt(INI_USER, INI_USER_SKIN,          m_eSkin);
	theApp.WriteProfileInt(INI_USER, INI_USER_TRANSPARENCY,  m_nTransparency);
}

///////////////////////////////////////////////////////////////////////////////
BOOL CXCPClockDlg::SetTransparentColor(COLORREF rgb, BOOL bTrans, BYTE transparency)
{
	if (!g_pSetLayeredWindowAttributes)
		return FALSE;
	
	if (bTrans)
	{
		// set layered style for the dialog
		SetWindowLong(m_hWnd, GWL_EXSTYLE, GetWindowLong(m_hWnd, GWL_EXSTYLE) | WS_EX_LAYERED);

		// call it with 0 alpha for the given color
		g_pSetLayeredWindowAttributes(m_hWnd, rgb, transparency, LWA_COLORKEY|LWA_ALPHA);
	}
	else
	{
		SetWindowLong(m_hWnd, GWL_EXSTYLE, GetWindowLong(m_hWnd, GWL_EXSTYLE) & ~WS_EX_LAYERED);
		
		// Ask the window and its children to repaint
		::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_FRAME | RDW_ALLCHILDREN);
	}
	
	return TRUE;
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::CenterRectOnMonitor(CRect& rect)
{
	int dlgw = rect.Width();
	int dlgh = rect.Height();

	int monw = GetSystemMetrics(SM_CXSCREEN);
	int monh = GetSystemMetrics(SM_CYSCREEN);

	CPoint centerpoint(monw/2, monh/2);

	HMONITOR hMon = MonitorFromWindow(m_hWnd, MONITOR_DEFAULTTONEAREST);

	if (hMon)
	{
		MONITORINFO mi;

		mi.cbSize = sizeof(mi);
		GetMonitorInfo(hMon, &mi);
		CRect r = mi.rcMonitor;

		centerpoint.x = r.left + r.Width()/2;
		centerpoint.y = r.top + r.Height()/2;
	}

	rect.left   = centerpoint.x - dlgw/2;
	rect.right  = rect.left + dlgw;
	rect.top    = centerpoint.y - dlgh/2;
	rect.bottom = rect.top + dlgh;
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

///////////////////////////////////////////////////////////////////////////////
HCURSOR CXCPClockDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnRButtonUp(UINT nFlags, CPoint point) 
{
	ShowPopup(point);
	
	CDialog::OnRButtonUp(nFlags, point);
}

///////////////////////////////////////////////////////////////////////////////
//
// The purpse of this function is to send a left mouse click to the
// underlying window.
//
BOOL CALLBACK EnumWindowsProc(HWND hWnd,		// handle to window
							  LPARAM lParam)	// HWND of caller
{
	HWND me = (HWND) lParam;

	if (hWnd == me)
	{
		// this is my window - ignore
		return TRUE;
	}

	CRect rect;
	::GetWindowRect(hWnd, &rect);

	if (rect.PtInRect(g_pointScreen))
	{
		// found window with the point that was clicked
		TCHAR szBuf[100];
		::SendMessage(hWnd, WM_GETTEXT, 50, (LPARAM)szBuf);
		szBuf[50] = 0;
		TRACE(_T("over window '%s'\n"), szBuf);

		HWND hParent = ::GetParent(hWnd);

		// loop to find top-most parent
		while (hParent)
		{
			HWND h = ::GetParent(hParent);
			if (h)
				hParent = h;
			else
				break;
		}

		if (hParent == 0)
			hParent = hWnd;

		TRACE(_T("hParent=%X\n"), hParent);

		CPoint pointClient(g_pointScreen);
		::ScreenToClient(hParent, &pointClient);

		// temporarily hide the window, so mouse click will go to correct window
		::ShowWindow(me, SW_HIDE);

		INPUT input[2] = 
		{
			INPUT_MOUSE, { pointClient.x, pointClient.y, 0, MOUSEEVENTF_LEFTDOWN, 0, 0},
			INPUT_MOUSE, { pointClient.x, pointClient.y, 0, MOUSEEVENTF_LEFTUP, 0, 0},
		};

		// send 2 left-button messages
		::SendInput(2, input, sizeof(INPUT));

		// must wait ...
		Sleep(100);

		// re-show window
		::ShowWindow(me, SW_SHOWNOACTIVATE);

		// stop enumeration
		return FALSE;
	}

	return TRUE;
}

///////////////////////////////////////////////////////////////////////////////
LRESULT CXCPClockDlg::OnXCPClockInit(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	TRACE(_T("in CXCPClockDlg::OnXCPClockInit: 0x%X\n"), m_hWnd);

	if (__argc > 1)
		return 15759;	// respond to broadcast message with a unique code,
						// if started with command line options
	else
		return 0;
}

///////////////////////////////////////////////////////////////////////////////
void CALLBACK SendAsyncProc(HWND hWnd,			// handle to destination window
							UINT uMsg,			// message
							ULONG_PTR dwData,	// application-defined value
							LRESULT lResult)	// result of message processing
{
	//TRACE(_T("in SendAsyncProc:  hWnd=0x%X  lResult=%d\n"), hWnd, lResult);

	HWND me = (HWND) dwData;

	if ((uMsg == WM_XCPCLOCK_INIT) && (hWnd != me) && (lResult == 15759))
	{
		// this callback is from a XCPClock app
		TRACE(_T("adding 0x%X\n"), hWnd);
		g_hWndArray.Add(hWnd);
	}
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnLButtonDown(UINT nFlags, CPoint point) 
{
	TRACE(_T("in CXCPClockDlg::OnLButtonDown\n"));
	m_bMouseCaptured = TRUE;

	SetCapture();
	
	m_ptLButtonDown = point;

	GetCursorPos(&g_pointScreen);

	static BOOL bFirstTime = TRUE;
	if (bFirstTime)
	{
		// collect HWNDs of other XCPClock apps
		TRACE(_T("calling SendMessageCallback\n"));
		::SendMessageCallback(HWND_BROADCAST, WM_XCPCLOCK_INIT, 0, 0, 
			SendAsyncProc, (ULONG_PTR) m_hWnd);
	}
	bFirstTime = FALSE;

	CDialog::OnLButtonDown(nFlags, point);
}


///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnLButtonUp(UINT nFlags, CPoint point) 
{
	TRACE(_T("in CXCPClockDlg::OnLButtonUp\n"));

	EndCapture();

	// if not fully opaque, allow click-thru
	if (m_nTransparency != 255)
	{
		CPoint pt;
		GetCursorPos(&pt);

		if (pt == g_pointScreen)
		{
			TRACE(_T("same point\n"));
			::EnumWindows(EnumWindowsProc, (LPARAM)m_hWnd);
		}
	}

	CDialog::OnLButtonUp(nFlags, point);
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::EndCapture()
{
	if (m_bMouseCaptured)
	{
		TRACE(_T("releasing mouse\n"));
		::ReleaseCapture();
	}
	m_bMouseCaptured = FALSE;
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnMouseMove(UINT nFlags, CPoint point) 
{
	if (m_bMouseCaptured)
	{
		//TRACE(_T("in CXCPClockDlg::OnMouseMove - mouse captured\n"));
		if (!theApp.m_strToolTip.IsEmpty())
			m_ToolTip.Hide();

		CRect rectDlg;
		GetWindowRect(&rectDlg);
		int w = rectDlg.Width();
		int h = rectDlg.Height();
		rectDlg.left  += point.x - m_ptLButtonDown.x;
		rectDlg.right  = rectDlg.left + w;
		rectDlg.top   += point.y - m_ptLButtonDown.y;
		rectDlg.bottom = rectDlg.top + h;
		MoveWindow(&rectDlg);

		// don't move other apps if Ctrl key is down
		if ((__argc > 1) && (GetAsyncKeyState(VK_CONTROL) >= 0))
		{
			// we were started from command line, so move other XCPClock apps too

			TRACE(_T("moving other apps\n"));

			for (int i = 0; i < g_hWndArray.GetSize(); i++)
			{
				HWND hWnd = (HWND) g_hWndArray.GetAt(i);

				if (hWnd && IsWindow(hWnd))
				{
					CRect rect;
					::GetWindowRect(hWnd, &rect);
					int w = rect.Width();
					int h = rect.Height();
					rect.left  += point.x - m_ptLButtonDown.x;
					rect.right  = rectDlg.left + w;
					rect.top   += point.y - m_ptLButtonDown.y;
					rect.bottom = rectDlg.top + h;
					::MoveWindow(hWnd, rect.left, rect.top, w, h, FALSE);
				}
			}
		}
	}
	else if (!theApp.m_strToolTip.IsEmpty())
	{
		CPoint pt;
		::GetCursorPos(&pt);
		m_ToolTip.Show(theApp.m_strToolTip, pt.x, pt.y);
	}

	CDialog::OnMouseMove(nFlags, point);
}

///////////////////////////////////////////////////////////////////////////////
BOOL CXCPClockDlg::OnEraseBkgnd(CDC* pDC) 
{
	// fill background with transparent color
	CRect rect;
	GetWindowRect(&rect);
	ScreenToClient(&rect);
	CBrush brush(RGB(255,0,255));
	pDC->FillRect(&rect, &brush);
	return TRUE;//CDialog::OnEraseBkgnd(pDC);
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnClose() 
{
	CDialog::OnCancel();	
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnCancel() 
{
	// don't let ESC key close the dialog
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnDestroy() 
{
	// don't save options if started with command line args
	if (__argc <= 1)
	{
		SaveWindowPos(m_hWnd, INI_PROGRAM_MAIN_WINDOW);
		SaveConfig();
	}

	CDialog::OnDestroy();
}

///////////////////////////////////////////////////////////////////////////////
BOOL CALLBACK EnumWindowsProcCloseAll(HWND hWnd,	// handle to window
									 LPARAM lParam)	// HWND of caller
{
	HWND me = (HWND) lParam;

	if (hWnd == me)
	{
		// this is my window - ignore
		return TRUE;
	}

	TCHAR szClassName[100];
	::GetClassName(hWnd, szClassName, sizeof(szClassName)/sizeof(TCHAR)-1);
	szClassName[sizeof(szClassName)/sizeof(TCHAR)-1] = 0;

	if (_tcsicmp(szClassName, _T("TheCodeProjectClock")) == 0)
	{
		TRACE(_T("closing window 0x%X\n"), hWnd);
		::SendMessage(::GetParent(hWnd), WM_CLOSE, 0, 0);
	}

	return TRUE;
}

///////////////////////////////////////////////////////////////////////////////
void CXCPClockDlg::OnCloseAll() 
{
	TRACE(_T("in CXCPClockDlg::OnCloseAll\n"));
	::EnumWindows(EnumWindowsProcCloseAll, (LPARAM)m_hWnd);
	GetParent()->SendMessage(WM_CLOSE);		// close this instance
}

///////////////////////////////////////////////////////////////////////////////
BOOL CXCPClockDlg::PreTranslateMessage(MSG* pMsg) 
{
	//TRACE(_T("in CXCPClockDlg::PreTranslateMessage\n"));
	TranslateAccelerator(m_hWnd, m_hAccel, pMsg);
	return CDialog::PreTranslateMessage(pMsg);
}

///////////////////////////////////////////////////////////////////////////////
BOOL CXCPClockDlg::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
	TRACE(_T("in CXCPClockDlg::OnMouseWheel\n"));
	if (nFlags & MK_CONTROL)
	{
		// Ctrl key is down
		TRACE(_T("ctrl key down\n"));
		if (zDelta < 0)
			OnIncreaseTransparency();
		else
			OnDecreaseTransparency();
	}
	return CDialog::OnMouseWheel(nFlags, zDelta, pt);
}
